@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2017 Intel Corporation.
+** Copyright (C) 2019 Intel Corporation.
 ** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
@@ -48,9 +48,7 @@
 
 #include <errno.h>
 
-#if QT_CONFIG(getentropy)
-#  include <sys/random.h>
-#elif !defined(Q_OS_BSD4) && !defined(Q_OS_WIN)
+#if !QT_CONFIG(getentropy) && (!defined(Q_OS_BSD4) || defined(__GLIBC__)) && !defined(Q_OS_WIN)
 #  include "qdeadlinetimer.h"
 #  include "qhashfunctions.h"
 
@@ -74,7 +72,7 @@ DECLSPEC_IMPORT BOOLEAN WINAPI SystemFunction036(PVOID RandomBuffer, ULONG Rando
 }
 #endif
 
-#if defined(Q_OS_ANDROID)
+#if defined(Q_OS_ANDROID) && !defined(Q_OS_ANDROID_EMBEDDED)
 #  include <private/qjni_p.h>
 #endif
 
@@ -92,42 +90,6 @@ DECLSPEC_IMPORT BOOLEAN WINAPI SystemFunction036(PVOID RandomBuffer, ULONG Rando
 
 QT_BEGIN_NAMESPACE
 
-#if defined(Q_PROCESSOR_X86) && QT_COMPILER_SUPPORTS_HERE(RDRND)
-static qsizetype qt_random_cpu(void *buffer, qsizetype count) Q_DECL_NOTHROW;
-
-#  ifdef Q_PROCESSOR_X86_64
-#    define _rdrandXX_step _rdrand64_step
-#  else
-#    define _rdrandXX_step _rdrand32_step
-#  endif
-
-static QT_FUNCTION_TARGET(RDRND) qsizetype qt_random_cpu(void *buffer, qsizetype count) Q_DECL_NOTHROW
-{
-    unsigned *ptr = reinterpret_cast<unsigned *>(buffer);
-    unsigned *end = ptr + count;
-
-    while (ptr + sizeof(qregisteruint)/sizeof(*ptr) <= end) {
-        if (_rdrandXX_step(reinterpret_cast<qregisteruint *>(ptr)) == 0)
-            goto out;
-        ptr += sizeof(qregisteruint)/sizeof(*ptr);
-    }
-
-    if (sizeof(*ptr) != sizeof(qregisteruint) && ptr != end) {
-        if (_rdrand32_step(ptr))
-            goto out;
-        ++ptr;
-    }
-
-out:
-    return ptr - reinterpret_cast<unsigned *>(buffer);
-}
-#else
-static qsizetype qt_random_cpu(void *, qsizetype)
-{
-    return 0;
-}
-#endif
-
 enum {
     // may be "overridden" by a member enum
     FillBufferNoexcept = true
@@ -136,7 +98,7 @@ enum {
 struct QRandomGenerator::SystemGenerator
 {
 #if QT_CONFIG(getentropy)
-    static qsizetype fillBuffer(void *buffer, qsizetype count) Q_DECL_NOTHROW
+    static qsizetype fillBuffer(void *buffer, qsizetype count) noexcept
     {
         // getentropy can read at most 256 bytes, so break the reading
         qsizetype read = 0;
@@ -188,7 +150,7 @@ struct QRandomGenerator::SystemGenerator
 #endif
     static void closeDevice()
     {
-        int fd = self().fdp1.load() - 1;
+        int fd = self().fdp1.loadRelaxed() - 1;
         if (fd >= 0)
             qt_safe_close(fd);
     }
@@ -206,13 +168,13 @@ struct QRandomGenerator::SystemGenerator
     }
 
 #elif defined(Q_OS_WIN) && !defined(Q_OS_WINRT)
-    qsizetype fillBuffer(void *buffer, qsizetype count) Q_DECL_NOTHROW
+    qsizetype fillBuffer(void *buffer, qsizetype count) noexcept
     {
         auto RtlGenRandom = SystemFunction036;
         return RtlGenRandom(buffer, ULONG(count)) ? count: 0;
     }
 #elif defined(Q_OS_WINRT)
-    qsizetype fillBuffer(void *, qsizetype) Q_DECL_NOTHROW
+    qsizetype fillBuffer(void *, qsizetype) noexcept
     {
         // always use the fallback
         return 0;
@@ -220,7 +182,8 @@ struct QRandomGenerator::SystemGenerator
 #endif // Q_OS_WINRT
 
     static SystemGenerator &self();
-    void generate(quint32 *begin, quint32 *end) Q_DECL_NOEXCEPT_EXPR(FillBufferNoexcept);
+    typedef quint32 result_type;
+    void generate(quint32 *begin, quint32 *end) noexcept(FillBufferNoexcept);
 
     // For std::mersenne_twister_engine implementations that use something
     // other than quint32 (unsigned int) to fill their buffers.
@@ -243,7 +206,7 @@ struct QRandomGenerator::SystemGenerator
 
 #if defined(Q_OS_WIN)
 static void fallback_update_seed(unsigned) {}
-static void fallback_fill(quint32 *ptr, qsizetype left) Q_DECL_NOTHROW
+static void fallback_fill(quint32 *ptr, qsizetype left) noexcept
 {
     // on Windows, rand_s is a high-quality random number generator
     // and it requires no seeding
@@ -255,14 +218,14 @@ static void fallback_fill(quint32 *ptr, qsizetype left) Q_DECL_NOTHROW
 }
 #elif QT_CONFIG(getentropy)
 static void fallback_update_seed(unsigned) {}
-static void fallback_fill(quint32 *, qsizetype) Q_DECL_NOTHROW
+static void fallback_fill(quint32 *, qsizetype) noexcept
 {
     // no fallback necessary, getentropy cannot fail under normal circumstances
     Q_UNREACHABLE();
 }
-#elif defined(Q_OS_BSD4)
+#elif defined(Q_OS_BSD4) && !defined(__GLIBC__)
 static void fallback_update_seed(unsigned) {}
-static void fallback_fill(quint32 *ptr, qsizetype left) Q_DECL_NOTHROW
+static void fallback_fill(quint32 *ptr, qsizetype left) noexcept
 {
     // BSDs have arc4random(4) and these work even in chroot(2)
     arc4random_buf(ptr, left * sizeof(*ptr));
@@ -281,7 +244,7 @@ Q_NEVER_INLINE
 #ifdef Q_CC_GNU
 __attribute__((cold))   // this function is pretty big, so optimize for size
 #endif
-static void fallback_fill(quint32 *ptr, qsizetype left) Q_DECL_NOTHROW
+static void fallback_fill(quint32 *ptr, qsizetype left) noexcept
 {
     quint32 scratch[12];    // see element count below
     quint32 *end = scratch;
@@ -311,7 +274,7 @@ static void fallback_fill(quint32 *ptr, qsizetype left) Q_DECL_NOTHROW
     *end++ = quint32(nsecs);    // 5
 #endif
 
-    if (quint32 v = seed.load())
+    if (quint32 v = seed.loadRelaxed())
         *end++ = v; // 6
 
 #if QT_CONFIG(getauxval)
@@ -355,22 +318,22 @@ static void fallback_fill(quint32 *ptr, qsizetype left) Q_DECL_NOTHROW
 #endif
 
 Q_NEVER_INLINE void QRandomGenerator::SystemGenerator::generate(quint32 *begin, quint32 *end)
-    Q_DECL_NOEXCEPT_EXPR(FillBufferNoexcept)
+    noexcept(FillBufferNoexcept)
 {
     quint32 *buffer = begin;
     qsizetype count = end - begin;
 
-    if (Q_UNLIKELY(uint(qt_randomdevice_control) & SetRandomData)) {
-        uint value = uint(qt_randomdevice_control) & RandomDataMask;
+    if (Q_UNLIKELY(uint(qt_randomdevice_control.loadAcquire()) & SetRandomData)) {
+        uint value = uint(qt_randomdevice_control.loadAcquire()) & RandomDataMask;
         std::fill_n(buffer, count, value);
         return;
     }
 
     qsizetype filled = 0;
-    if (qt_has_hwrng() && (uint(qt_randomdevice_control) & SkipHWRNG) == 0)
-        filled += qt_random_cpu(buffer, count);
+    if (qHasHwrng() && (uint(qt_randomdevice_control.loadAcquire()) & SkipHWRNG) == 0)
+        filled += qRandomCpu(buffer, count);
 
-    if (filled != count && (uint(qt_randomdevice_control) & SkipSystemRNG) == 0) {
+    if (filled != count && (uint(qt_randomdevice_control.loadAcquire()) & SkipSystemRNG) == 0) {
         qsizetype bytesFilled =
                 fillBuffer(buffer + filled, (count - filled) * qsizetype(sizeof(*buffer)));
         filled += bytesFilled / qsizetype(sizeof(*buffer));
@@ -512,9 +475,7 @@ inline QRandomGenerator::SystemGenerator &QRandomGenerator::SystemGenerator::sel
     those. The most common way of generating new values is to call the generate(),
     generate64() or fillRange() functions. One would use it as:
 
-    \code
-        quint32 value = QRandomGenerator::global()->generate();
-    \endcode
+    \snippet code/src_corelib_global_qrandom.cpp 0
 
     Additionally, it provides a floating-point function generateDouble() that
     returns a number in the range [0, 1) (that is, inclusive of zero and
@@ -527,11 +488,7 @@ inline QRandomGenerator::SystemGenerator &QRandomGenerator::SystemGenerator::sel
     the numbers generated by the object will always be the same, as in the
     following example:
 
-    \code
-        QRandomGenerator prng1(1234), prng2(1234);
-        Q_ASSERT(prng1.generate32() == prng2.generate32());
-        Q_ASSERT(prng1.generate64() == prng2.generate64());
-    \endcode
+    \snippet code/src_corelib_global_qrandom.cpp 1
 
     The seed data takes the form of one or more 32-bit words. The ideal seed
     size is approximately equal to the size of the QRandomGenerator class
@@ -554,12 +511,7 @@ inline QRandomGenerator::SystemGenerator &QRandomGenerator::SystemGenerator::sel
     For ease of use, QRandomGenerator provides a global object that can
     be easily used, as in the following example:
 
-    \code
-        int x = QRandomGenerator::global()->generate32();
-        int y = QRandomGenerator::global()->generate32();
-        int w = QRandomGenerator::global()->bounded(16384);
-        int h = QRandomGenerator::global()->bounded(16384);
-    \endcode
+    \snippet code/src_corelib_global_qrandom.cpp 2
 
     \section1 System-wide random number generator
 
@@ -647,14 +599,16 @@ inline QRandomGenerator::SystemGenerator &QRandomGenerator::SystemGenerator::sel
     following code may be used to generate a floating-point number in the range
     [1, 2.5):
 
-    \code
-        std::uniform_real_distribution dist(1, 2.5);
-        return dist(*QRandomGenerator::global());
-    \endcode
+    \snippet code/src_corelib_global_qrandom.cpp 3
 
     \sa QRandomGenerator64, qrand()
  */
 
+/*!
+    \enum QRandomGenerator::System
+    \internal
+*/
+
 /*!
     \fn QRandomGenerator::QRandomGenerator(quint32 seedValue)
 
@@ -666,7 +620,7 @@ inline QRandomGenerator::SystemGenerator &QRandomGenerator::SystemGenerator::sel
  */
 
 /*!
-    \fn QRandomGenerator::QRandomGenerator(const quint32 (&seedBuffer)[N])
+    \fn template <qsizetype N> QRandomGenerator::QRandomGenerator(const quint32 (&seedBuffer)[N])
     \overload
 
     Initializes this QRandomGenerator object with the values found in the
@@ -685,16 +639,13 @@ inline QRandomGenerator::SystemGenerator &QRandomGenerator::SystemGenerator::sel
     with the same seed value will produce the same number sequence.
 
     This constructor is equivalent to:
-    \code
-        std::seed_seq sseq(seedBuffer, seedBuffer + len);
-        QRandomGenerator generator(sseq);
-    \endcode
+    \snippet code/src_corelib_global_qrandom.cpp 4
 
     \sa seed(), securelySeeded()
  */
 
 /*!
-    \fn QRandomGenerator::QRandomGenerator(const quint32 *begin, const quin32 *end)
+    \fn QRandomGenerator::QRandomGenerator(const quint32 *begin, const quint32 *end)
     \overload
 
     Initializes this QRandomGenerator object with the values found in the range
@@ -702,10 +653,7 @@ inline QRandomGenerator::SystemGenerator &QRandomGenerator::SystemGenerator::sel
     with the same seed value will produce the same number sequence.
 
     This constructor is equivalent to:
-    \code
-        std::seed_seq sseq(begin, end);
-        QRandomGenerator generator(sseq);
-    \endcode
+    \snippet code/src_corelib_global_qrandom.cpp 5
 
     \sa seed(), securelySeeded()
  */
@@ -760,9 +708,9 @@ inline QRandomGenerator::SystemGenerator &QRandomGenerator::SystemGenerator::sel
 /*!
     \typedef QRandomGenerator::result_type
 
-    A typedef to the type that operator()() returns. That is, quint32.
+    A typedef to the type that operator() returns. That is, quint32.
 
-    \sa operator()()
+    \sa operator()
  */
 
 /*!
@@ -773,6 +721,22 @@ inline QRandomGenerator::SystemGenerator &QRandomGenerator::SystemGenerator::sel
     \sa generate(), generate64()
  */
 
+/*!
+    \fn quint32 QRandomGenerator::generate()
+
+    Generates a 32-bit random quantity and returns it.
+
+    \sa {QRandomGenerator::operator()}{operator()()}, generate64()
+ */
+
+/*!
+    \fn quint64 QRandomGenerator::generate64()
+
+    Generates a 64-bit random quantity and returns it.
+
+    \sa {QRandomGenerator::operator()}{operator()()}, generate()
+ */
+
 /*!
     \fn result_type QRandomGenerator::min()
 
@@ -800,7 +764,7 @@ inline QRandomGenerator::SystemGenerator &QRandomGenerator::SystemGenerator::sel
     \fn void QRandomGenerator::seed(std::seed_seq &seed)
     \overload
 
-    Reseeds this object using the seed sequence \a sseq as the seed.
+    Reseeds this object using the seed sequence \a seed as the seed.
  */
 
 /*!
@@ -809,24 +773,19 @@ inline QRandomGenerator::SystemGenerator &QRandomGenerator::SystemGenerator::sel
     Discards the next \a z entries from the sequence. This method is equivalent
     to calling generate() \a z times and discarding the result, as in:
 
-    \code
-        while (z--)
-            generator.generate();
-    \endcode
+    \snippet code/src_corelib_global_qrandom.cpp 6
 */
 
 /*!
-    \fn void QRandomGenerator::generate(ForwardIterator begin, ForwardIterator end)
+    \fn template <typename ForwardIterator> void QRandomGenerator::generate(ForwardIterator begin, ForwardIterator end)
 
     Generates 32-bit quantities and stores them in the range between \a begin
     and \a end. This function is equivalent to (and is implemented as):
 
-    \code
-        std::generate(begin, end, [this]() { return generate(); });
-    \endcode
+    \snippet code/src_corelib_global_qrandom.cpp 7
 
     This function complies with the requirements for the function
-    \c{\l{http://en.cppreference.com/w/cpp/numeric/random/seed_seq/generate}{std::seed_seq::generate}},
+    \l{http://en.cppreference.com/w/cpp/numeric/random/seed_seq/generate}{\c std::seed_seq::generate},
     which requires unsigned 32-bit integer values.
 
     Note that if the [begin, end) range refers to an area that can store more
@@ -834,9 +793,7 @@ inline QRandomGenerator::SystemGenerator &QRandomGenerator::SystemGenerator::sel
     32 bits of data. Any other bits will be zero. To fill the range with 64 bit
     quantities, one can write:
 
-    \code
-        std::generate(begin, end, []() { return QRandomGenerator::global()->generate64(); });
-    \endcode
+    \snippet code/src_corelib_global_qrandom.cpp 8
 
     If the range refers to contiguous memory (such as an array or the data from
     a QVector), the fillRange() function may be used too.
@@ -853,7 +810,7 @@ inline QRandomGenerator::SystemGenerator &QRandomGenerator::SystemGenerator::sel
  */
 
 /*!
-    \fn void QRandomGenerator::fillRange(UInt *buffer, qsizetype count)
+    \fn template <typename UInt> void QRandomGenerator::fillRange(UInt *buffer, qsizetype count)
 
     Generates \a count 32- or 64-bit quantities (depending on the type \c UInt)
     and stores them in the buffer pointed by \a buffer. This is the most
@@ -863,17 +820,13 @@ inline QRandomGenerator::SystemGenerator &QRandomGenerator::SystemGenerator::sel
     For example, to fill a vector of 16 entries with random values, one may
     write:
 
-    \code
-        QVector<quint32> vector;
-        vector.resize(16);
-        QRandomGenerator::fillRange(vector.data(), vector.size());
-    \endcode
+    \snippet code/src_corelib_global_qrandom.cpp 9
 
     \sa generate()
  */
 
 /*!
-    \fn void QRandomGenerator::fillRange(UInt (&buffer)[N})
+    \fn template <typename UInt, size_t N> void QRandomGenerator::fillRange(UInt (&buffer)[N])
 
     Generates \c N 32- or 64-bit quantities (depending on the type \c UInt) and
     stores them in the \a buffer array. This is the most efficient way to
@@ -882,10 +835,7 @@ inline QRandomGenerator::SystemGenerator &QRandomGenerator::SystemGenerator::sel
 
     For example, to fill generate two 32-bit quantities, one may write:
 
-    \code
-        quint32 array[2];
-        QRandomGenerator::fillRange(array);
-    \endcode
+    \snippet code/src_corelib_global_qrandom.cpp 10
 
     It would have also been possible to make one call to generate64() and then split
     the two halves of the 64-bit value.
@@ -900,27 +850,26 @@ inline QRandomGenerator::SystemGenerator &QRandomGenerator::SystemGenerator::sel
     inclusive of zero and exclusive of 1).
 
     This function is equivalent to:
-    \code
-        QRandomGenerator64 rd;
-        return std::generate_canonical<qreal, std::numeric_limits<qreal>::digits>(rd);
-    \endcode
+    \snippet code/src_corelib_global_qrandom.cpp 11
 
     The same may also be obtained by using
-    \c{\l{http://en.cppreference.com/w/cpp/numeric/random/uniform_real_distribution}{std::uniform_real_distribution}}
+    \l{http://en.cppreference.com/w/cpp/numeric/random/uniform_real_distribution}{\c std::uniform_real_distribution}
     with parameters 0 and 1.
 
     \sa generate(), generate64(), bounded()
  */
 
 /*!
-    \fn qreal QRandomGenerator::bounded(qreal highest)
+    \fn double QRandomGenerator::bounded(double highest)
 
-    Generates one random qreal in the range between 0 (inclusive) and \a
+    Generates one random double in the range between 0 (inclusive) and \a
     highest (exclusive). This function is equivalent to and is implemented as:
 
-    \code
-        return generateDouble() * highest;
-    \endcode
+    \snippet code/src_corelib_global_qrandom.cpp 12
+
+    If the \a highest parameter is negative, the result will be negative too;
+    if it is infinite or NaN, the result will be infinite or NaN too (that is,
+    not random).
 
     \sa generateDouble(), bounded()
  */
@@ -931,15 +880,13 @@ inline QRandomGenerator::SystemGenerator &QRandomGenerator::SystemGenerator::sel
 
     Generates one random 32-bit quantity in the range between 0 (inclusive) and
     \a highest (exclusive). The same result may also be obtained by using
-    \c{\l{http://en.cppreference.com/w/cpp/numeric/random/uniform_int_distribution}{std::uniform_int_distribution}}
+    \l{http://en.cppreference.com/w/cpp/numeric/random/uniform_int_distribution}{\c std::uniform_int_distribution}
     with parameters 0 and \c{highest - 1}. That class can also be used to obtain
     quantities larger than 32 bits.
 
     For example, to obtain a value between 0 and 255 (inclusive), one would write:
 
-    \code
-        quint32 v = QRandomGenerator::bounded(256);
-    \endcode
+    \snippet code/src_corelib_global_qrandom.cpp 13
 
     Naturally, the same could also be obtained by masking the result of generate()
     to only the lower 8 bits. Either solution is as efficient.
@@ -951,11 +898,11 @@ inline QRandomGenerator::SystemGenerator &QRandomGenerator::SystemGenerator::sel
  */
 
 /*!
-    \fn quint32 QRandomGenerator::bounded(int highest)
+    \fn int QRandomGenerator::bounded(int highest)
     \overload
 
     Generates one random 32-bit quantity in the range between 0 (inclusive) and
-    \a highest (exclusive). \a highest must not be negative.
+    \a highest (exclusive). \a highest must be positive.
 
     Note that this function cannot be used to obtain values in the full 32-bit
     range of int. Instead, use generate() and cast to int.
@@ -967,19 +914,19 @@ inline QRandomGenerator::SystemGenerator &QRandomGenerator::SystemGenerator::sel
     \fn quint32 QRandomGenerator::bounded(quint32 lowest, quint32 highest)
     \overload
 
-    Generates one random 32-bit quantity in the range between \a lowest (inclusive)
-    and \a highest (exclusive). The same result may also be obtained by using
-    \c{\l{http://en.cppreference.com/w/cpp/numeric/random/uniform_int_distribution}{std::uniform_int_distribution}}
+    Generates one random 32-bit quantity in the range between \a lowest
+    (inclusive) and \a highest (exclusive). The \a highest parameter must be
+    greater than \a lowest.
+
+    The same result may also be obtained by using
+    \l{http://en.cppreference.com/w/cpp/numeric/random/uniform_int_distribution}{\c std::uniform_int_distribution}
     with parameters \a lowest and \c{\a highest - 1}. That class can also be used to
     obtain quantities larger than 32 bits.
 
     For example, to obtain a value between 1000 (incl.) and 2000 (excl.), one
     would write:
 
-    \code
-        quint32 v = QRandomGenerator::bounded(1000, 2000);
-    \endcode
-
+    \snippet code/src_corelib_global_qrandom.cpp 14
 
     Note that this function cannot be used to obtain values in the full 32-bit
     range of quint32. Instead, use generate().
@@ -988,11 +935,12 @@ inline QRandomGenerator::SystemGenerator &QRandomGenerator::SystemGenerator::sel
  */
 
 /*!
-    \fn quint32 QRandomGenerator::bounded(int lowest, int highest)
+    \fn int QRandomGenerator::bounded(int lowest, int highest)
     \overload
 
     Generates one random 32-bit quantity in the range between \a lowest
-    (inclusive) and \a highest (exclusive), both of which may be negative.
+    (inclusive) and \a highest (exclusive), both of which may be negative, but
+    \a highest must be greater than \a lowest.
 
     Note that this function cannot be used to obtain values in the full 32-bit
     range of int. Instead, use generate() and cast to int.
@@ -1034,9 +982,7 @@ inline QRandomGenerator::SystemGenerator &QRandomGenerator::SystemGenerator::sel
 
     For example, the following creates a random RGB color:
 
-    \code
-        return QColor::fromRgb(QRandomGenerator::global()->generate());
-    \endcode
+    \snippet code/src_corelib_global_qrandom.cpp 15
 
     Accesses to this object are thread-safe and it may therefore be used in any
     thread without locks. The object may also be copied and the sequence
@@ -1086,19 +1032,12 @@ inline QRandomGenerator::SystemGenerator &QRandomGenerator::SystemGenerator::sel
     \sa QRandomGenerator
 */
 
-/*!
-   \fn QRandomGenerator64::QRandomGenerator64(const QRandomGenerator &other)
-   \internal
-
-   Creates a copy.
-*/
-
 /*!
     \typedef QRandomGenerator64::result_type
 
-    A typedef to the type that operator()() returns. That is, quint64.
+    A typedef to the type that operator() returns. That is, quint64.
 
-    \sa operator()()
+    \sa operator()
  */
 
 /*!
@@ -1111,9 +1050,7 @@ inline QRandomGenerator::SystemGenerator &QRandomGenerator::SystemGenerator::sel
     set. If you wish to cast the returned value to qint64 and keep it positive,
     you should mask the sign bit off:
 
-    \code
-        qint64 value = QRandomGenerator64::generate() & std::numeric_limits<qint64>::max();
-    \endcode
+    \snippet code/src_corelib_global_qrandom.cpp 16
 
     \sa QRandomGenerator, QRandomGenerator::generate64()
  */
@@ -1169,7 +1106,9 @@ QRandomGenerator64 QRandomGenerator64::securelySeeded()
     return result;
 }
 
-/// \internal
+/*!
+    \internal
+*/
 inline QRandomGenerator::QRandomGenerator(System)
     : type(SystemRNG)
 {
@@ -1200,7 +1139,7 @@ QRandomGenerator &QRandomGenerator::operator=(const QRandomGenerator &other)
     return *this;
 }
 
-QRandomGenerator::QRandomGenerator(std::seed_seq &sseq) Q_DECL_NOTHROW
+QRandomGenerator::QRandomGenerator(std::seed_seq &sseq) noexcept
     : type(MersenneTwister)
 {
     Q_ASSERT(this != system());
@@ -1255,27 +1194,63 @@ void QRandomGenerator::_fillRange(void *buffer, void *bufferEnd)
     quint32 *begin = static_cast<quint32 *>(buffer);
     quint32 *end = static_cast<quint32 *>(bufferEnd);
 
-    if (type == SystemRNG || Q_UNLIKELY(uint(qt_randomdevice_control) & (UseSystemRNG|SetRandomData)))
+    if (type == SystemRNG || Q_UNLIKELY(uint(qt_randomdevice_control.loadAcquire()) & (UseSystemRNG|SetRandomData)))
         return SystemGenerator::self().generate(begin, end);
 
     SystemAndGlobalGenerators::PRNGLocker lock(this);
     std::generate(begin, end, [this]() { return storage.engine()(); });
 }
 
-#if defined(Q_OS_ANDROID) && (__ANDROID_API__ < 21)
-typedef QThreadStorage<QJNIObjectPrivate> AndroidRandomStorage;
-Q_GLOBAL_STATIC(AndroidRandomStorage, randomTLS)
+namespace {
+struct QRandEngine
+{
+    std::minstd_rand engine;
+    QRandEngine() : engine(1) {}
 
-#elif defined(Q_OS_UNIX) && !defined(QT_NO_THREAD) && defined(_POSIX_THREAD_SAFE_FUNCTIONS) && (_POSIX_THREAD_SAFE_FUNCTIONS - 0 > 0)
-using SeedStorageType = QtPrivate::FunctionPointer<decltype(&srand)>::Arguments::Car;
+    int generate()
+    {
+        std::minstd_rand::result_type v = engine();
+        if (std::numeric_limits<int>::max() != RAND_MAX)
+            v %= uint(RAND_MAX) + 1;
 
-typedef QThreadStorage<SeedStorageType *> SeedStorage;
-Q_GLOBAL_STATIC(SeedStorage, randTLS)  // Thread Local Storage for seed value
+        return int(v);
+    }
 
+    void seed(std::minstd_rand::result_type q)
+    {
+        engine.seed(q);
+    }
+};
+}
+
+#if defined(Q_OS_WIN)
+// On Windows srand() and rand() already use Thread-Local-Storage
+// to store the seed between calls
+static inline QRandEngine *randTLS()
+{
+    return nullptr;
+}
+#elif defined(Q_COMPILER_THREAD_LOCAL)
+static inline QRandEngine *randTLS()
+{
+    thread_local QRandEngine r;
+    return &r;
+}
+#else
+Q_GLOBAL_STATIC(QThreadStorage<QRandEngine>, g_randTLS)
+static inline QRandEngine *randTLS()
+{
+    auto tls = g_randTLS();
+    if (!tls)
+        return nullptr;
+    return &tls->localData();
+
+}
 #endif
 
 /*!
     \relates <QtGlobal>
+    \deprecated
     \since 4.2
 
     Thread-safe version of the standard C++ \c srand() function.
@@ -1287,49 +1262,23 @@ Q_GLOBAL_STATIC(SeedStorage, randTLS)  // Thread Local Storage for seed value
     if two threads call qsrand(1) and subsequently call qrand(), the threads will get
     the same random number sequence.
 
+    \note This function is deprecated. In new applications, use
+    QRandomGenerator instead.
+
     \sa qrand(), QRandomGenerator
 */
 void qsrand(uint seed)
 {
-#if defined(Q_OS_ANDROID) && (__ANDROID_API__ < 21)
-    if (randomTLS->hasLocalData()) {
-        randomTLS->localData().callMethod<void>("setSeed", "(J)V", jlong(seed));
-        return;
-    }
-
-    QJNIObjectPrivate random("java/util/Random",
-                             "(J)V",
-                             jlong(seed));
-    if (!random.isValid()) {
+    auto prng = randTLS();
+    if (prng)
+        prng->seed(seed);
+    else
         srand(seed);
-        return;
-    }
-
-    randomTLS->setLocalData(random);
-#elif defined(Q_OS_UNIX) && !defined(QT_NO_THREAD) && defined(_POSIX_THREAD_SAFE_FUNCTIONS) && (_POSIX_THREAD_SAFE_FUNCTIONS - 0 > 0)
-    SeedStorage *seedStorage = randTLS();
-    if (seedStorage) {
-        SeedStorageType *pseed = seedStorage->localData();
-        if (!pseed)
-            seedStorage->setLocalData(pseed = new SeedStorageType);
-        *pseed = seed;
-    } else {
-        //global static seed storage should always exist,
-        //except after being deleted by QGlobalStaticDeleter.
-        //But since it still can be called from destructor of another
-        //global static object, fallback to srand(seed)
-        srand(seed);
-    }
-#else
-    // On Windows srand() and rand() already use Thread-Local-Storage
-    // to store the seed between calls
-    // this is also valid for QT_NO_THREAD
-    srand(seed);
-#endif
 }
 
 /*!
     \relates <QtGlobal>
+    \deprecated
     \since 4.2
 
     Thread-safe version of the standard C++ \c rand() function.
@@ -1343,52 +1292,18 @@ void qsrand(uint seed)
     step is skipped, then the sequence will be pre-seeded with a constant
     value.
 
-    \sa qsrand(), QRandomGenerator
+    \note This function is deprecated. In new applications, use
+    QRandomGenerator instead.
+
+    \sa qrand(), QRandomGenerator
 */
 int qrand()
 {
-#if defined(Q_OS_ANDROID) && (__ANDROID_API__ < 21)
-    AndroidRandomStorage *randomStorage = randomTLS();
-    if (!randomStorage)
+    auto prng = randTLS();
+    if (prng)
+        return prng->generate();
+    else
         return rand();
-
-    if (randomStorage->hasLocalData()) {
-        return randomStorage->localData().callMethod<jint>("nextInt",
-                                                           "(I)I",
-                                                           RAND_MAX);
-    }
-
-    QJNIObjectPrivate random("java/util/Random",
-                             "(J)V",
-                             jlong(1));
-
-    if (!random.isValid())
-        return rand();
-
-    randomStorage->setLocalData(random);
-    return random.callMethod<jint>("nextInt", "(I)I", RAND_MAX);
-#elif defined(Q_OS_UNIX) && !defined(QT_NO_THREAD) && defined(_POSIX_THREAD_SAFE_FUNCTIONS) && (_POSIX_THREAD_SAFE_FUNCTIONS - 0 > 0)
-    SeedStorage *seedStorage = randTLS();
-    if (seedStorage) {
-        SeedStorageType *pseed = seedStorage->localData();
-        if (!pseed) {
-            seedStorage->setLocalData(pseed = new SeedStorageType);
-            *pseed = 1;
-        }
-        return rand_r(pseed);
-    } else {
-        //global static seed storage should always exist,
-        //except after being deleted by QGlobalStaticDeleter.
-        //But since it still can be called from destructor of another
-        //global static object, fallback to rand()
-        return rand();
-    }
-#else
-    // On Windows srand() and rand() already use Thread-Local-Storage
-    // to store the seed between calls
-    // this is also valid for QT_NO_THREAD
-    return rand();
-#endif
 }
 
 QT_END_NAMESPACE
