@@ -211,6 +211,12 @@ QDebug operator<<(QDebug debug, const QMacAutoReleasePool *pool)
     debug << "QMacAutoReleasePool(" << (const void *)pool << ')';
     return debug;
 }
+
+QDebug operator<<(QDebug debug, const QCFString &string)
+{
+    debug << static_cast<QString>(string);
+    return debug;
+}
 #endif // !QT_NO_DEBUG_STREAM
 
 #ifdef Q_OS_MACOS
@@ -371,7 +377,7 @@ bool operator<(const KeyPair &entry, const Qt::Key &key)
 struct qtKey2CocoaKeySortLessThan
 {
     typedef bool result_type;
-    Q_DECL_CONSTEXPR result_type operator()(const KeyPair &entry1, const KeyPair &entry2) const Q_DECL_NOTHROW
+    Q_DECL_CONSTEXPR result_type operator()(const KeyPair &entry1, const KeyPair &entry2) const noexcept
     {
         return entry1.qtKey < entry2.qtKey;
     }
@@ -501,7 +507,7 @@ void qt_apple_check_os_version()
         if (!applicationName)
             applicationName = NSProcessInfo.processInfo.processName;
 
-        fprintf(stderr, "Sorry, \"%s\" can not be run on this version of %s. "
+        fprintf(stderr, "Sorry, \"%s\" cannot be run on this version of %s. "
             "Qt requires %s %ld.%ld.%ld or later, you have %s %ld.%ld.%ld.\n",
             applicationName.UTF8String, os,
             os, long(required.majorVersion), long(required.minorVersion), long(required.patchVersion),
@@ -514,5 +520,35 @@ Q_CONSTRUCTOR_FUNCTION(qt_apple_check_os_version);
 
 // -------------------------------------------------------------------------
 
+void QMacKeyValueObserver::addObserver(NSKeyValueObservingOptions options)
+{
+    [object addObserver:observer forKeyPath:keyPath options:options context:callback.get()];
+}
+
+void QMacKeyValueObserver::removeObserver() {
+    if (object)
+        [object removeObserver:observer forKeyPath:keyPath context:callback.get()];
+    object = nil;
+}
+
+KeyValueObserver *QMacKeyValueObserver::observer = [[KeyValueObserver alloc] init];
+
+QT_END_NAMESPACE
+@implementation KeyValueObserver
+- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object
+        change:(NSDictionary<NSKeyValueChangeKey, id> *)change context:(void *)context
+{
+    Q_UNUSED(keyPath);
+    Q_UNUSED(object);
+    Q_UNUSED(change);
+
+    (*reinterpret_cast<QMacKeyValueObserver::Callback*>(context))();
+}
+@end
+QT_BEGIN_NAMESPACE
+
+// -------------------------------------------------------------------------
+
+
 QT_END_NAMESPACE
 
