@@ -84,9 +84,12 @@
 #include <qpa/qplatformscreen.h>
 #include <private/qguiapplication_p.h>
 #include <private/qhighdpiscaling_p.h>
+#include <private/qwidget_p.h>
 
 #include <private/qstylehelper_p.h>
+#if QT_CONFIG(animation)
 #include <private/qstyleanimation_p.h>
+#endif
 
 #include <algorithm>
 
@@ -117,10 +120,7 @@ enum QSliderDirection { SlUp, SlDown, SlLeft, SlRight };
     \internal
 */
 
-QWindowsStylePrivate::QWindowsStylePrivate()
-    : alt_down(false), menuBarTimer(0)
-{
-}
+QWindowsStylePrivate::QWindowsStylePrivate() = default;
 
 qreal QWindowsStylePrivate::appDevicePixelRatio()
 {
@@ -154,7 +154,7 @@ bool QWindowsStyle::eventFilter(QObject *o, QEvent *e)
             QList<QWidget *> l = widget->findChildren<QWidget *>();
             auto ignorable = [](QWidget *w) {
                 return w->isWindow() || !w->isVisible()
-                        || w->style()->styleHint(SH_UnderlineShortcut, 0, w);
+                        || w->style()->styleHint(SH_UnderlineShortcut, nullptr, w);
             };
             l.erase(std::remove_if(l.begin(), l.end(), ignorable), l.end());
             // Update states before repainting
@@ -239,14 +239,15 @@ void QWindowsStyle::polish(QApplication *app)
     QCommonStyle::polish(app);
     QWindowsStylePrivate *d = const_cast<QWindowsStylePrivate*>(d_func());
     // We only need the overhead when shortcuts are sometimes hidden
-    if (!proxy()->styleHint(SH_UnderlineShortcut, 0) && app)
+    if (!proxy()->styleHint(SH_UnderlineShortcut, nullptr) && app)
         app->installEventFilter(this);
 
-    d->activeCaptionColor = app->palette().highlight().color();
-    d->activeGradientCaptionColor = app->palette().highlight() .color();
-    d->inactiveCaptionColor = app->palette().dark().color();
-    d->inactiveGradientCaptionColor = app->palette().dark().color();
-    d->inactiveCaptionText = app->palette().background().color();
+    const auto &palette = QGuiApplication::palette();
+    d->activeGradientCaptionColor = palette.highlight().color();
+    d->activeCaptionColor = d->activeGradientCaptionColor;
+    d->inactiveGradientCaptionColor = palette.dark().color();
+    d->inactiveCaptionColor = d->inactiveGradientCaptionColor;
+    d->inactiveCaptionText = palette.window().color();
 
 #if defined(Q_OS_WIN) && !defined(Q_OS_WINRT) //fetch native title bar colors
     if(app->desktopSettingsAware()){
@@ -339,7 +340,6 @@ int QWindowsStylePrivate::fixedPixelMetric(QStyle::PixelMetric pm)
     case QStyle::PM_MenuVMargin:
     case QStyle::PM_ToolBarItemMargin:
         return 1;
-        break;
     case QStyle::PM_DockWidgetSeparatorExtent:
         return 4;
 #if QT_CONFIG(tabbar)
@@ -378,23 +378,12 @@ int QWindowsStylePrivate::fixedPixelMetric(QStyle::PixelMetric pm)
     return QWindowsStylePrivate::InvalidMetric;
 }
 
-static QWindow *windowOf(const QWidget *w)
-{
-    QWindow *result = nullptr;
-    if (w) {
-        result = w->windowHandle();
-        if (!result) {
-            if (const QWidget *np = w->nativeParentWidget())
-                result = np->windowHandle();
-        }
-    }
-    return result;
-}
-
 static QScreen *screenOf(const QWidget *w)
 {
-    if (const QWindow *window = windowOf(w))
-        return window->screen();
+    if (w) {
+        if (auto screen = qt_widget_private(const_cast<QWidget *>(w))->associatedScreen())
+            return screen;
+    }
     return QGuiApplication::primaryScreen();
 }
 
@@ -403,8 +392,6 @@ static QScreen *screenOf(const QWidget *w)
 // and account for secondary screens with differing logical DPI.
 qreal QWindowsStylePrivate::nativeMetricScaleFactor(const QWidget *widget)
 {
-    if (!QHighDpiScaling::isActive())
-        return 1;
     qreal result = qreal(1) / QWindowsStylePrivate::devicePixelRatio(widget);
     if (QGuiApplicationPrivate::screen_list.size() > 1) {
         const QScreen *primaryScreen = QGuiApplication::primaryScreen();
@@ -430,7 +417,7 @@ int QWindowsStyle::pixelMetric(PixelMetric pm, const QStyleOption *opt, const QW
 
     ret = QWindowsStylePrivate::fixedPixelMetric(pm);
     if (ret != QWindowsStylePrivate::InvalidMetric)
-        return int(QStyleHelper::dpiScaled(ret));
+        return int(QStyleHelper::dpiScaled(ret, opt));
 
     ret = 0;
 
@@ -476,7 +463,7 @@ int QWindowsStyle::pixelMetric(PixelMetric pm, const QStyleOption *opt, const QW
         break;
 
     case PM_SplitterWidth:
-        ret = qMax(int(QStyleHelper::dpiScaled(4)), QApplication::globalStrut().width());
+        ret = qMax(int(QStyleHelper::dpiScaled(4, opt)), QApplication::globalStrut().width());
         break;
 
     default:
@@ -552,7 +539,7 @@ int QWindowsStyle::styleHint(StyleHint hint, const QStyleOption *opt, const QWid
     case SH_MenuBar_MouseTracking:
     case SH_Menu_MouseTracking:
     case SH_ComboBox_ListMouseTracking:
-    case SH_ScrollBar_StopMouseOverSlider:
+    case SH_Slider_StopMouseOverSlider:
     case SH_MainWindow_SpaceBelowMenuBar:
         ret = 1;
 
@@ -705,17 +692,17 @@ void QWindowsStyle::drawPrimitive(PrimitiveElement pe, const QStyleOption *opt,
                 x -= 2;
             if (opt->rect.height() > 4) {
                 qDrawShadePanel(p, x, 2, 3, opt->rect.height() - 4,
-                                opt->palette, false, 1, 0);
+                                opt->palette, false, 1, nullptr);
                 qDrawShadePanel(p, x + 3, 2, 3, opt->rect.height() - 4,
-                                opt->palette, false, 1, 0);
+                                opt->palette, false, 1, nullptr);
             }
         } else {
             if (opt->rect.width() > 4) {
                 int y = opt->rect.height() / 2 - 4;
                 qDrawShadePanel(p, 2, y, opt->rect.width() - 4, 3,
-                                opt->palette, false, 1, 0);
+                                opt->palette, false, 1, nullptr);
                 qDrawShadePanel(p, 2, y + 3, opt->rect.width() - 4, 3,
-                                opt->palette, false, 1, 0);
+                                opt->palette, false, 1, nullptr);
             }
         }
         p->restore();
@@ -766,7 +753,7 @@ void QWindowsStyle::drawPrimitive(PrimitiveElement pe, const QStyleOption *opt,
                 }
             } else {
                 qDrawWinButton(p, opt->rect, opt->palette,
-                               opt->state & (State_Sunken | State_On), panel ? &fill : 0);
+                               opt->state & (State_Sunken | State_On), panel ? &fill : nullptr);
             }
         } else {
             p->fillRect(opt->rect, fill);
@@ -800,8 +787,11 @@ void QWindowsStyle::drawPrimitive(PrimitiveElement pe, const QStyleOption *opt,
         QPen oldPen = p->pen();
         p->setPen(QPen(opt->palette.shadow().color(), 0));
         QRectF rect = opt->rect;
-        rect.adjust(QStyleHelper::dpiScaled(0.5), QStyleHelper::dpiScaled(0.5),
-                    QStyleHelper::dpiScaled(-1.5), QStyleHelper::dpiScaled(-1.5));
+        const qreal dpi = QStyleHelper::dpi(opt);
+        const qreal topLevelAdjustment = QStyleHelper::dpiScaled(0.5, dpi);
+        const qreal bottomRightAdjustment = QStyleHelper::dpiScaled(-1.5, dpi);
+        rect.adjust(topLevelAdjustment, topLevelAdjustment,
+                    bottomRightAdjustment, bottomRightAdjustment);
         p->drawRect(rect);
         p->setPen(oldPen);
         break;
@@ -815,7 +805,7 @@ void QWindowsStyle::drawPrimitive(PrimitiveElement pe, const QStyleOption *opt,
         else if (opt->state & State_Enabled)
             fill = opt->palette.base();
         else
-            fill = opt->palette.background();
+            fill = opt->palette.window();
         p->save();
         doRestore = true;
         qDrawWinPanel(p, opt->rect, opt->palette, true, &fill);
@@ -825,13 +815,13 @@ void QWindowsStyle::drawPrimitive(PrimitiveElement pe, const QStyleOption *opt,
             p->setPen(opt->palette.text().color());
         }
         Q_FALLTHROUGH();
-    case PE_IndicatorViewItemCheck:
+    case PE_IndicatorItemViewItemCheck:
         if (!doRestore) {
             p->save();
             doRestore = true;
         }
 #if QT_CONFIG(itemviews)
-        if (pe == PE_IndicatorViewItemCheck) {
+        if (pe == PE_IndicatorItemViewItemCheck) {
             const QStyleOptionViewItem *itemViewOpt = qstyleoption_cast<const QStyleOptionViewItem *>(opt);
             p->setPen(itemViewOpt
                       && itemViewOpt->showDecorationSelected
@@ -840,17 +830,19 @@ void QWindowsStyle::drawPrimitive(PrimitiveElement pe, const QStyleOption *opt,
                         : opt->palette.text().color());
             if (opt->state & State_NoChange)
                 p->setBrush(opt->palette.brush(QPalette::Button));
-            p->drawRect(opt->rect.x() + 1, opt->rect.y() + 1, 11, 11);
+            p->drawRect(opt->rect.x() + 1, opt->rect.y() + 1, opt->rect.width() - 2, opt->rect.height() - 2);
         }
 #endif // QT_CONFIG(itemviews)
         if (!(opt->state & State_Off)) {
             QPointF  points[6];
-            points[0] = { opt->rect.x() + QStyleHelper::dpiScaled(3.5), opt->rect.y() + QStyleHelper::dpiScaled(5.5) };
-            points[1] = { points[0].x(), points[0].y() + QStyleHelper::dpiScaled(2) };
-            points[2] = { points[1].x() + QStyleHelper::dpiScaled(2), points[1].y() + QStyleHelper::dpiScaled(2) };
-            points[3] = { points[2].x() + QStyleHelper::dpiScaled(4), points[2].y() - QStyleHelper::dpiScaled(4) };
-            points[4] = { points[3].x(), points[3].y() - QStyleHelper::dpiScaled(2) };
-            points[5] = { points[4].x() - QStyleHelper::dpiScaled(4), points[4].y() + QStyleHelper::dpiScaled(4) };
+            qreal scaleh = opt->rect.width() / 12.0;
+            qreal scalev = opt->rect.height() / 12.0;
+            points[0] = { opt->rect.x() + 3.5 * scaleh, opt->rect.y() + 5.5 * scalev };
+            points[1] = { points[0].x(),                points[0].y() + 2 * scalev };
+            points[2] = { points[1].x() + 2 * scaleh,   points[1].y() + 2 * scalev };
+            points[3] = { points[2].x() + 4 * scaleh,   points[2].y() - 4 * scalev };
+            points[4] = { points[3].x(),                points[3].y() - 2 * scalev };
+            points[5] = { points[4].x() - 4 * scaleh,   points[4].y() + 4 * scalev };
             p->setPen(QPen(opt->palette.text().color(), 0));
             p->setBrush(opt->palette.text().color());
             p->drawPolygon(points, 6);
@@ -947,14 +939,14 @@ void QWindowsStyle::drawPrimitive(PrimitiveElement pe, const QStyleOption *opt,
             if (frame->lineWidth == 2 || pe == PE_Frame) {
                 QPalette popupPal = frame->palette;
                 if (pe == PE_FrameMenu) {
-                    popupPal.setColor(QPalette::Light, frame->palette.background().color());
+                    popupPal.setColor(QPalette::Light, frame->palette.window().color());
                     popupPal.setColor(QPalette::Midlight, frame->palette.light().color());
                 }
                 if (pe == PE_Frame && (frame->state & State_Raised))
                     qDrawWinButton(p, frame->rect, popupPal, frame->state & State_Sunken);
                 else if (pe == PE_Frame && (frame->state & State_Sunken))
                 {
-                    popupPal.setColor(QPalette::Midlight, frame->palette.background().color());
+                    popupPal.setColor(QPalette::Midlight, frame->palette.window().color());
                     qDrawWinPanel(p, frame->rect, popupPal, frame->state & State_Sunken);
                 }
                 else
@@ -964,7 +956,7 @@ void QWindowsStyle::drawPrimitive(PrimitiveElement pe, const QStyleOption *opt,
             }
         } else {
             QPalette popupPal = opt->palette;
-            popupPal.setColor(QPalette::Light, opt->palette.background().color());
+            popupPal.setColor(QPalette::Light, opt->palette.window().color());
             popupPal.setColor(QPalette::Midlight, opt->palette.light().color());
             qDrawWinPanel(p, opt->rect, popupPal, opt->state & State_Sunken);
         }
@@ -982,7 +974,7 @@ void QWindowsStyle::drawPrimitive(PrimitiveElement pe, const QStyleOption *opt,
 
         if (opt->state & (State_Raised | State_On | State_Sunken)) {
             qDrawWinButton(p, opt->rect, opt->palette, opt->state & (State_Sunken | State_On),
-                           panel ? &fill : 0);
+                           panel ? &fill : nullptr);
         } else {
             if (panel)
                 p->fillRect(opt->rect, fill);
@@ -992,7 +984,7 @@ void QWindowsStyle::drawPrimitive(PrimitiveElement pe, const QStyleOption *opt,
         break; }
     case PE_FrameWindow: {
          QPalette popupPal = opt->palette;
-         popupPal.setColor(QPalette::Light, opt->palette.background().color());
+         popupPal.setColor(QPalette::Light, opt->palette.window().color());
          popupPal.setColor(QPalette::Midlight, opt->palette.light().color());
          qDrawWinPanel(p, opt->rect, popupPal, opt->state & State_Sunken);
         break; }
@@ -1007,7 +999,7 @@ void QWindowsStyle::drawPrimitive(PrimitiveElement pe, const QStyleOption *opt,
 #endif // QT_CONFIG(dockwidget)
 
     case PE_FrameStatusBarItem:
-        qDrawShadePanel(p, opt->rect, opt->palette, true, 1, 0);
+        qDrawShadePanel(p, opt->rect, opt->palette, true, 1, nullptr);
         break;
 
     case PE_IndicatorProgressChunk:
@@ -1045,7 +1037,7 @@ void QWindowsStyle::drawPrimitive(PrimitiveElement pe, const QStyleOption *opt,
         break;
 
     case PE_FrameTabWidget: {
-        qDrawWinButton(p, opt->rect, opt->palette, false, 0);
+        qDrawWinButton(p, opt->rect, opt->palette, false, nullptr);
         break;
     }
     default:
@@ -1148,9 +1140,6 @@ void QWindowsStyle::drawControl(ControlElement ce, const QStyleOption *opt, QPai
                     pixmap = menuitem->icon.pixmap(proxy()->pixelMetric(PM_SmallIconSize, opt, widget), mode);
                 const int pixw = pixmap.width() / pixmap.devicePixelRatio();
                 const int pixh = pixmap.height() / pixmap.devicePixelRatio();
-                if (act && !dis && !checked)
-                    qDrawShadePanel(p, vCheckRect,  menuitem->palette, false, 1,
-                                    &menuitem->palette.brush(QPalette::Button));
                 QRect pmr(0, 0, pixw, pixh);
                 pmr.moveCenter(vCheckRect.center());
                 p->setPen(menuitem->palette.text().color());
@@ -1314,12 +1303,12 @@ void QWindowsStyle::drawControl(ControlElement ce, const QStyleOption *opt, QPai
                     x2 -= onlyOne || lastTab ? borderThinkness : 0;
                 }
 
-                p->fillRect(QRect(x1 + 1, y1 + 1, (x2 - x1) - 1, (y2 - y1) - 2), tab->palette.background());
+                p->fillRect(QRect(x1 + 1, y1 + 1, (x2 - x1) - 1, (y2 - y1) - 2), tab->palette.window());
 
                 // Delete border
                 if (selected) {
-                    p->fillRect(QRect(x1,y2-1,x2-x1,1), tab->palette.background());
-                    p->fillRect(QRect(x1,y2,x2-x1,1), tab->palette.background());
+                    p->fillRect(QRect(x1,y2-1,x2-x1,1), tab->palette.window());
+                    p->fillRect(QRect(x1,y2,x2-x1,1), tab->palette.window());
                 }
                 // Left
                 if (firstTab || selected || onlyOne || !previousSelected) {
@@ -1350,12 +1339,12 @@ void QWindowsStyle::drawControl(ControlElement ce, const QStyleOption *opt, QPai
                     x2 -= lastTab ? borderThinkness : 0;
                 }
 
-                p->fillRect(QRect(x1 + 1, y1 + 2, (x2 - x1) - 1, (y2 - y1) - 1), tab->palette.background());
+                p->fillRect(QRect(x1 + 1, y1 + 2, (x2 - x1) - 1, (y2 - y1) - 1), tab->palette.window());
 
                 // Delete border
                 if (selected) {
-                    p->fillRect(QRect(x1, y1 + 1, (x2 - 1)-x1, 1), tab->palette.background());
-                    p->fillRect(QRect(x1, y1, (x2 - 1)-x1, 1), tab->palette.background());
+                    p->fillRect(QRect(x1, y1 + 1, (x2 - 1)-x1, 1), tab->palette.window());
+                    p->fillRect(QRect(x1, y1, (x2 - 1)-x1, 1), tab->palette.window());
                 }
                 // Left
                 if (firstTab || selected || onlyOne || !previousSelected) {
@@ -1388,12 +1377,12 @@ void QWindowsStyle::drawControl(ControlElement ce, const QStyleOption *opt, QPai
                     y2 -= lastTab ? borderThinkness : 0;
                 }
 
-                p->fillRect(QRect(x1 + 1, y1 + 1, (x2 - x1) - 2, (y2 - y1) - 1), tab->palette.background());
+                p->fillRect(QRect(x1 + 1, y1 + 1, (x2 - x1) - 2, (y2 - y1) - 1), tab->palette.window());
 
                 // Delete border
                 if (selected) {
-                    p->fillRect(QRect(x2 - 1, y1, 1, y2-y1), tab->palette.background());
-                    p->fillRect(QRect(x2, y1, 1, y2-y1), tab->palette.background());
+                    p->fillRect(QRect(x2 - 1, y1, 1, y2-y1), tab->palette.window());
+                    p->fillRect(QRect(x2, y1, 1, y2-y1), tab->palette.window());
                 }
                 // Top
                 if (firstTab || selected || onlyOne || !previousSelected) {
@@ -1426,12 +1415,12 @@ void QWindowsStyle::drawControl(ControlElement ce, const QStyleOption *opt, QPai
                     y2 -= lastTab ? borderThinkness : 0;
                 }
 
-                p->fillRect(QRect(x1 + 2, y1 + 1, (x2 - x1) - 1, (y2 - y1) - 1), tab->palette.background());
+                p->fillRect(QRect(x1 + 2, y1 + 1, (x2 - x1) - 1, (y2 - y1) - 1), tab->palette.window());
 
                 // Delete border
                 if (selected) {
-                    p->fillRect(QRect(x1 + 1, y1, 1, (y2 - 1)-y1),tab->palette.background());
-                    p->fillRect(QRect(x1, y1, 1, (y2-1)-y1), tab->palette.background());
+                    p->fillRect(QRect(x1 + 1, y1, 1, (y2 - 1)-y1),tab->palette.window());
+                    p->fillRect(QRect(x1, y1, 1, (y2-1)-y1), tab->palette.window());
                 }
                 // Top
                 if (firstTab || selected || onlyOne || !previousSelected) {
@@ -1525,7 +1514,7 @@ void QWindowsStyle::drawControl(ControlElement ce, const QStyleOption *opt, QPai
                         br = QBrush(paletteBrush.textureImage());
                 } else
                     br = QBrush(opt->palette.light().color(), Qt::Dense4Pattern);
-                p->setBackground(opt->palette.background().color());
+                p->setBackground(opt->palette.window().color());
                 p->setBrush(br);
             }
             p->drawRect(opt->rect);
@@ -1590,6 +1579,7 @@ void QWindowsStyle::drawControl(ControlElement ce, const QStyleOption *opt, QPai
                 case QStyleOptionToolBar::Beginning:
                 case QStyleOptionToolBar::OnlyOne:
                     paintBottomBorder = false;
+                    break;
                 default:
                     break;
                 }
@@ -1605,6 +1595,7 @@ void QWindowsStyle::drawControl(ControlElement ce, const QStyleOption *opt, QPai
                 case QStyleOptionToolBar::OnlyOne:
                     paintRightBorder = false;
                     paintLeftBorder = false;
+                    break;
                 default:
                     break;
                 }
@@ -1692,7 +1683,7 @@ void QWindowsStyle::drawControl(ControlElement ce, const QStyleOption *opt, QPai
             }
             QPalette pal2 = pb->palette;
             // Correct the highlight color if it is the same as the background
-            if (pal2.highlight() == pal2.background())
+            if (pal2.highlight() == pal2.window())
                 pal2.setColor(QPalette::Highlight, pb->palette.color(QPalette::Active,
                                                                      QPalette::Highlight));
             bool reverse = ((!vertical && (pb->direction == Qt::RightToLeft)) || vertical);
@@ -1824,7 +1815,7 @@ void QWindowsStyle::drawControl(ControlElement ce, const QStyleOption *opt, QPai
                                     titleRect.height(), titleRect.width());
                 }
                 proxy()->drawItemText(p, titleRect,
-                            Qt::AlignLeft | Qt::AlignVCenter | Qt::TextShowMnemonic, palette,
+                            Qt::AlignLeft | Qt::AlignVCenter, palette,
                             dwOpt->state & State_Enabled, dwOpt->title,
                             floating ? (active ? QPalette::BrightText : QPalette::Window) : QPalette::WindowText);
                 p->setFont(oldFont);
@@ -1842,7 +1833,7 @@ void QWindowsStyle::drawControl(ControlElement ce, const QStyleOption *opt, QPai
                 p->setBackground(cb->palette.highlight());
             } else {
                 p->setPen(cb->palette.text().color());
-                p->setBackground(cb->palette.background());
+                p->setBackground(cb->palette.window());
             }
         }
         QCommonStyle::drawControl(ce, opt, p, widget);
@@ -2184,7 +2175,7 @@ void QWindowsStyle::drawComplexControl(ComplexControl cc, const QStyleOptionComp
 
                 } else {
                     p->setPen(cmb->palette.text().color());
-                    p->setBackground(cmb->palette.background());
+                    p->setBackground(cmb->palette.window());
                 }
 
                 if (cmb->state & State_HasFocus && !cmb->editable) {
@@ -2309,8 +2300,9 @@ QSize QWindowsStyle::sizeFromContents(ContentsType ct, const QStyleOption *opt,
             int defwidth = 0;
             if (btn->features & QStyleOptionButton::AutoDefaultButton)
                 defwidth = 2 * proxy()->pixelMetric(PM_ButtonDefaultIndicator, btn, widget);
-            int minwidth = int(QStyleHelper::dpiScaled(75.));
-            int minheight = int(QStyleHelper::dpiScaled(23.));
+            const qreal dpi = QStyleHelper::dpi(opt);
+            int minwidth = int(QStyleHelper::dpiScaled(75, dpi));
+            int minheight = int(QStyleHelper::dpiScaled(23, dpi));
 
 #ifndef QT_QWS_SMALL_PUSHBUTTON
             if (w < minwidth + defwidth && !btn->text.isEmpty())
