@@ -599,7 +599,7 @@ void tst_QSplitter::testShowHide()
     QWidget widget(&topLevel);
     widget.resize(400 + split->handleWidth(), 200);
     QVBoxLayout *lay=new QVBoxLayout(&widget);
-    lay->setMargin(0);
+    lay->setContentsMargins(0, 0, 0, 0);
     lay->setSpacing(0);
     split->addWidget(new QWidget);
     split->addWidget(new QWidget);
@@ -735,6 +735,7 @@ void tst_QSplitter::replaceWidget()
 
     // Configure splitter
     QWidget *oldWidget = sp.widget(index);
+    QTest::qWait(100); // Make sure we record the right original size (after the window manager adds the frame)
     const QRect oldGeom = oldWidget ? oldWidget->geometry() : QRect();
     if (oldWidget) {
         // Collapse first, then hide, if necessary
@@ -773,12 +774,16 @@ void tst_QSplitter::replaceWidget()
         const int expectedResizeCount = visible ? 1 : 0; // new widget only
         const int expectedPaintCount = visible && !collapsed ? 2 : 0; // splitter and new widget
         QTRY_COMPARE(ef.resizeCount, expectedResizeCount);
+#ifndef Q_OS_WINRT // QTBUG-68297
         QTRY_COMPARE(ef.paintCount, expectedPaintCount);
+#endif
         QCOMPARE(newWidget->parentWidget(), &sp);
         QCOMPARE(newWidget->isVisible(), visible);
         if (visible && !collapsed)
             QCOMPARE(newWidget->geometry(), oldGeom);
+#ifndef Q_OS_WINRT // QTBUG-68297
         QCOMPARE(newWidget->size().isEmpty(), !visible || collapsed);
+#endif
         delete res;
     }
     QCOMPARE(sp.count(), count);
@@ -830,7 +835,9 @@ void tst_QSplitter::replaceWidgetWithSplitterChild()
         QTRY_VERIFY(ef.resizeCount > 0);
         QTRY_VERIFY(ef.paintCount > 0);
         QCOMPARE(sp.count(), count + 1);
+#ifndef Q_OS_WINRT // QTBUG-68297
         QCOMPARE(sp.sizes().mid(0, count), sizes);
+#endif
         QCOMPARE(sp.sizes().last(), sibling->width());
     } else {
         // No-op for the rest
